@@ -11,6 +11,9 @@ module Imports (
   , module Data.Word
   , module Data.Maybe
   , module Numeric
+  , GCBuffer
+  , withForeignPtr
+  , mallocPlainForeignPtrBytes
   ) where
 
 import Control.Applicative
@@ -24,4 +27,8 @@ import Data.Maybe
 import Data.Monoid
 import Data.Ord
 import Data.Word
+import Foreign.ForeignPtr
 import Numeric
+import GHC.ForeignPtr (mallocPlainForeignPtrBytes)
+
+type GCBuffer = ForeignPtr Word8
