@@ -1,12 +1,12 @@
 module Network.HPACK.Huffman (
-  -- * Type
-    HuffmanEncoding
-  , HuffmanDecoding
-  -- * Encoding/decoding
-  , encode
+  -- * Huffman encoding/decoding
+    encodeH
   , encodeHuffman
-  , decode
+  , decodeH
   , decodeHuffman
+  , HuffmanDecoder
+  , decH
+  , GCBuffer
   ) where
 
 import Network.HPACK.Huffman.Decode
