@@ -1,10 +1,12 @@
-{-# LANGUAGE BangPatterns, OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Network.HPACK.Huffman.Decode (
   -- * Huffman decoding
-    HuffmanDecoding
-  , decode
+    decodeH
   , decodeHuffman
+  , HuffmanDecoder
+  , decH
+  , GCBuffer
   ) where
 
 import Control.Exception (throwIO)
@@ -13,6 +15,7 @@ import Data.Array.Base (unsafeAt)
 import qualified Data.ByteString as BS
 import Network.ByteOrder
 
+import Imports
 import Network.HPACK.Huffman.Bit
 import Network.HPACK.Huffman.Params
 import Network.HPACK.Huffman.Table
@@ -22,20 +25,20 @@ import Network.HPACK.Types (DecodeError(..))
 ----------------------------------------------------------------
 
 -- | Huffman decoding.
-type HuffmanDecoding = ReadBuffer -> Int -> IO ByteString
+type HuffmanDecoder = ReadBuffer -> Int -> IO ByteString
 
 ----------------------------------------------------------------
 
 data Pin = EndOfString
-         | Forward {-# UNPACK #-} !Word8 -- node no.
-         | GoBack  {-# UNPACK #-} !Word8 -- node no.
-                   {-# UNPACK #-} !Word8 -- a decoded value
-         | GoBack2 {-# UNPACK #-} !Word8 -- node no.
-                   {-# UNPACK #-} !Word8 -- a decoded value
-                   {-# UNPACK #-} !Word8 -- a decoded value
+         | Forward {-# UNPACK #-} Word8 -- node no.
+         | GoBack  {-# UNPACK #-} Word8 -- node no.
+                   {-# UNPACK #-} Word8 -- a decoded value
+         | GoBack2 {-# UNPACK #-} Word8 -- node no.
+                   {-# UNPACK #-} Word8 -- a decoded value
+                   {-# UNPACK #-} Word8 -- a decoded value
          deriving Show
 
-data WayStep = WayStep !(Maybe Int) !(Array Word8 Pin)
+data WayStep = WayStep (Maybe Int) (Array Word8 Pin)
 type Way256 = Array Word8 WayStep
 
 next :: WayStep -> Word8 -> Pin
@@ -44,25 +47,30 @@ next (WayStep _ a16) w = a16 `unsafeAt` fromIntegral w
 ----------------------------------------------------------------
 
 -- | Huffman decoding.
-decode :: Buffer -> BufferSize -> HuffmanDecoding
-decode buf siz rbuf len = do
-    wbuf <- newWriteBuffer buf siz
-    dec wbuf rbuf len
+decodeH :: GCBuffer    -- ^ A working space
+        -> BufferSize
+        -> ReadBuffer  -- ^ A read buffer which contains the target
+        -> Int         -- ^ The target length
+        -> IO ByteString
+decodeH gcbuf bufsiz rbuf len = withForeignPtr gcbuf $ \buf -> do
+    wbuf <- newWriteBuffer buf bufsiz
+    decH wbuf rbuf len
     toByteString wbuf
 
-dec :: WriteBuffer -> ReadBuffer -> Int -> IO ()
-dec wbuf rbuf len = go len (way256 `unsafeAt` 0)
+-- | Low devel Huffman decoding in a write buffer.
+decH :: WriteBuffer -> ReadBuffer -> Int -> IO ()
+decH wbuf rbuf len = go len (way256 `unsafeAt` 0)
   where
     go 0 way0 = case way0 of
         WayStep Nothing  _ -> throwIO IllegalEos
         WayStep (Just i) _
           | i <= 8         -> return ()
           | otherwise      -> throwIO TooLongEos
-    go !n !way0 = do
+    go n way0 = do
         w <- read8 rbuf
         way <- doit way0 w
         go (n - 1) way
-    doit !way !w = case next way w of
+    doit way w = case next way w of
         EndOfString -> throwIO EosInTheMiddle
         Forward n   -> return $ way256 `unsafeAt` fromIntegral n
         GoBack  n v -> do
@@ -73,9 +81,10 @@ dec wbuf rbuf len = go len (way256 `unsafeAt` 0)
             write8 wbuf v2
             return $ way256 `unsafeAt` fromIntegral n
 
+-- | Huffman decoding with a temporary buffer whose size is 4096.
 decodeHuffman :: ByteString -> IO ByteString
 decodeHuffman bs = withWriteBuffer 4096 $ \wbuf ->
-    withReadBuffer bs $ \rbuf -> dec wbuf rbuf (BS.length bs)
+    withReadBuffer bs $ \rbuf -> decH wbuf rbuf $ BS.length bs
 
 ----------------------------------------------------------------
 
@@ -88,12 +97,12 @@ construct decoder = listArray (0,255) $ map to16ways $ flatten decoder
   where
     to16ways x = WayStep ei a16
       where
-        !ei = eosInfo x
-        !a16 = listArray (0,255) $ map (step decoder x Non) bits8s
+        ei = eosInfo x
+        a16 = listArray (0,255) $ map (step decoder x Non) bits8s
 
 data Chara = Non
-           | One !Word8
-           | Two !Word8 !Word8
+           | One Word8
+           | Two Word8 Word8
 
 inc :: Chara -> Word8 -> Chara
 inc Non w     = One w
@@ -103,8 +112,8 @@ inc _       _ = error "inc"
 step :: HTree -> HTree -> Chara -> [B] -> Pin
 step root (Tip _ v)     x  bss
   | v == idxEos                     = EndOfString
-  | otherwise                       = let !w = fromIntegral v
-                                          !x' = inc x w
+  | otherwise                       = let w = fromIntegral v
+                                          x' = inc x w
                                       in step root root x' bss
 step _    (Bin _ n _ _) Non       [] = Forward (fromIntegral n)
 step _    (Bin _ n _ _) (One w)   [] = GoBack (fromIntegral n) w
