@@ -1,4 +1,3 @@
-{-# LANGUAGE TupleSections #-}
 {-# OPTIONS_HADDOCK hide #-}
 
 module Network.HPACK.Table (
@@ -6,7 +5,6 @@ module Network.HPACK.Table (
     DynamicTable
   , newDynamicTableForEncoding
   , newDynamicTableForDecoding
-  , clearDynamicTable
   , withDynamicTableForEncoding
   , withDynamicTableForDecoding
   , huffmanDecoder
