@@ -1,4 +1,6 @@
-{-# LANGUAGE BangPatterns, OverloadedStrings, CPP, RecordWildCards #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards #-}
 
 module Network.HPACK.Table.RevIndex (
     RevIndex
@@ -27,7 +29,7 @@ import Network.HPACK.Types
 
 ----------------------------------------------------------------
 
-data RevIndex = RevIndex !DynamicRevIndex !OtherRevIdex
+data RevIndex = RevIndex DynamicRevIndex OtherRevIdex
 
 type DynamicRevIndex = Array Int (IORef ValueMap)
 
@@ -45,7 +47,7 @@ type OtherRevIdex = IORef (Map KeyValue HIndex)
 
 type StaticRevIndex = Array Int StaticEntry
 
-data StaticEntry = StaticEntry !HIndex !(Maybe ValueMap) deriving Show
+data StaticEntry = StaticEntry HIndex (Maybe ValueMap) deriving Show
 
 type ValueMap = Map HeaderValue HIndex
 
@@ -59,7 +61,7 @@ staticRevIndex = A.array (minTokenIx,maxStaticTokenIx) $ map toEnt zs
         m = case xs of
             []  -> error "staticRevIndex"
             [("",i)] -> StaticEntry i Nothing
-            (_,i):_  -> let !vs = M.fromList xs
+            (_,i):_  -> let vs = M.fromList xs
                         in StaticEntry i (Just vs)
     zs = map extract $ groupBy ((==) `on` fst) lst
       where
@@ -124,21 +126,19 @@ renewOtherRevIndex ref = writeIORef ref M.empty
 lookupOtherRevIndex :: Header -> OtherRevIdex -> (HIndex -> IO ()) -> IO () -> IO ()
 lookupOtherRevIndex (k,v) ref fa' fc' = do
       oth <- readIORef ref
-      case M.lookup (KeyValue k v) oth of
-          Just i  -> fa' i
-          Nothing -> fc'
+      maybe fc' fa' $ M.lookup (KeyValue k v) oth
 
 {-# INLINE insertOtherRevIndex #-}
 insertOtherRevIndex :: Token -> HeaderValue -> HIndex -> OtherRevIdex -> IO ()
 insertOtherRevIndex t v i ref = modifyIORef' ref $ M.insert (KeyValue k v) i
   where
-    !k = tokenFoldedKey t
+    k = tokenFoldedKey t
 
 {-# INLINE deleteOtherRevIndex #-}
 deleteOtherRevIndex :: Token -> HeaderValue -> OtherRevIdex -> IO ()
 deleteOtherRevIndex t v ref = modifyIORef' ref $ M.delete (KeyValue k v)
   where
-    !k = tokenFoldedKey t
+    k = tokenFoldedKey t
 
 ----------------------------------------------------------------
 
@@ -160,10 +160,10 @@ lookupRevIndex :: Token
                -> RevIndex
                -> IO ()
 lookupRevIndex t@Token{..} v fa fb fc fd (RevIndex dyn oth)
-  | not (isStaticTokenIx ix) = lookupOtherRevIndex (k,v) oth fa' fc'
-  | shouldBeIndexed          = lookupDynamicStaticRevIndex ix v dyn fa' fb'
+  | not (isStaticTokenIx tokenIx) = lookupOtherRevIndex (k,v) oth fa' fc'
+  | shouldBeIndexed               = lookupDynamicStaticRevIndex tokenIx v dyn fa' fb'
   -- path: is not indexed but ":path /" should be used, sigh.
-  | otherwise                = lookupStaticRevIndex ix v fa' fd'
+  | otherwise                     = lookupStaticRevIndex tokenIx v fa' fd'
   where
     k = foldedCase tokenKey
     ent = toEntryToken t v
@@ -180,8 +180,8 @@ lookupRevIndex' :: Token
                 -> (HeaderName -> HeaderValue -> IO ())
                 -> IO ()
 lookupRevIndex' Token{..} v fa fd fe
-  | isStaticTokenIx ix = lookupStaticRevIndex ix v fa' fd'
-  | otherwise          = fe'
+  | isStaticTokenIx tokenIx = lookupStaticRevIndex tokenIx v fa' fd'
+  | otherwise               = fe'
   where
     k = foldedCase tokenKey
     fa' = fa
