@@ -4,7 +4,6 @@ module Network.HPACK.Types (
   -- * Header
     HeaderName
   , HeaderValue
-  , HeaderStuff
   , Header
   , HeaderList
   , TokenHeader
@@ -44,9 +43,6 @@ type Header = (HeaderName, HeaderValue)
 -- | Header list.
 type HeaderList = [Header]
 
--- | To be a 'HeaderName' or 'HeaderValue'.
-type HeaderStuff = ByteString
-
 -- | TokenBased header.
 type TokenHeader = (Token, HeaderValue)
 
@@ -71,9 +67,9 @@ data CompressionAlgo = Naive  -- ^ No compression
 -- | Strategy for HPACK encoding.
 data EncodeStrategy = EncodeStrategy {
   -- | Which compression algorithm is used.
-    compressionAlgo :: !CompressionAlgo
+    compressionAlgo :: CompressionAlgo
   -- | Whether or not to use Huffman encoding for strings.
-  , useHuffman :: !Bool
+  , useHuffman :: Bool
   } deriving (Eq, Show)
 
 -- | Default 'EncodeStrategy'.
@@ -93,12 +89,12 @@ data DecodeError = IndexOverrun Index -- ^ Index is out of range
                  | EosInTheMiddle -- ^ Eos appears in the middle of huffman string
                  | IllegalEos -- ^ Non-eos appears in the end of huffman string
                  | TooLongEos -- ^ Eos of huffman string is more than 7 bits
-                 | EmptyEncodedString -- ^ Encoded string has no length
                  | TooSmallTableSize -- ^ A peer set the dynamic table size less than 32
                  | TooLargeTableSize -- ^ A peer tried to change the dynamic table size over the limit
                  | IllegalTableSizeUpdate -- ^ Table size update at the non-beginning
                  | HeaderBlockTruncated
                  | IllegalHeaderName
+                 | TooLargeHeader
                  deriving (Eq,Show,Typeable)
 
 instance Exception DecodeError
