@@ -1,4 +1,4 @@
-{-# LANGUAGE OverloadedStrings, RecordWildCards, CPP #-}
+{-# LANGUAGE RecordWildCards, CPP #-}
 
 module Case where
 
@@ -9,7 +9,7 @@ import Data.ByteString (ByteString)
 import qualified Data.ByteString.Base16 as B16
 
 import JSON
-import Network.HTTP2
+import Network.HTTP2.Frame
 
 data CaseSource = CaseSource {
     cs_description :: String
@@ -44,7 +44,7 @@ wireToCase CaseWire { wire_error = Nothing, ..} = Case {
   }
   where
     -- fromJust is unsafe
-    frm = case decodeFrame defaultSettings $ fst $ B16.decode wire_hex of
+    frm = case decodeFrame defaultSettings $ B16.decodeLenient wire_hex of
         Left  e -> error $ show e
         Right r -> r
 wireToCase CaseWire { wire_error = Just e, ..} = Case {
