@@ -1,4 +1,4 @@
-{-# LANGUAGE TypeSynonymInstances, FlexibleInstances #-}
+{-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE OverloadedStrings, RecordWildCards, CPP #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
@@ -14,12 +14,12 @@ import Data.Aeson.Types
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as B8
-import Data.HashMap.Strict (union)
-import Data.Maybe (fromJust, fromMaybe)
+import Data.Aeson.KeyMap (union)
+import Data.Maybe (fromJust)
 import Data.Text (Text)
 import qualified Data.Text as T
 
-import Network.HTTP2
+import Network.HTTP2.Frame
 
 ----------------------------------------------------------------
 
@@ -42,7 +42,7 @@ data FramePad = FramePad {
   , fpPad :: Maybe Pad
   } deriving (Show, Read)
 
-data Pad = Pad Padding deriving (Show, Read)
+newtype Pad = Pad Padding deriving (Show, Read)
 
 unPad :: Pad -> Padding
 unPad (Pad x) = x
@@ -91,9 +91,9 @@ instance ToJSON FramePayload where
         "data" .= body
       ]
     toJSON (HeadersFrame mpri hdr) = object [
-        "exclusive" .= fromMaybe Null (toJSON . exclusive <$> mpri)
-      , "stream_dependency" .= fromMaybe Null (toJSON . streamDependency <$> mpri)
-      , "weight" .= fromMaybe Null (toJSON . weight <$> mpri)
+        "exclusive"             .= maybe Null (toJSON . exclusive) mpri
+      , "stream_dependency"     .= maybe Null (toJSON . streamDependency) mpri
+      , "weight"                .= maybe Null (toJSON . weight) mpri
       , "header_block_fragment" .= hdr
       ]
     toJSON (PriorityFrame pri) = object [
