@@ -21,14 +21,14 @@ import Network.HPACK.Table
 
 import JSON
 
-data Conf = Conf {
+newtype Conf = Conf {
     debug :: Bool
   }
 
 data Result = Pass | Fail String deriving (Eq,Show)
 
 run :: Bool -> Test -> IO Result
-run _ (Test _ [])        = return $ Pass
+run _ (Test _ [])        = return Pass
 run d (Test _ ccs) = do
     -- 'size c' must not be used. Initial value is defaultDynamicTableSize!
     withDynamicTableForDecoding defaultDynamicTableSize 4096 $ \dyntbl -> do
@@ -39,7 +39,7 @@ testLoop :: Conf
          -> [Case]
          -> DynamicTable
          -> IO Result
-testLoop _    []     _      = return $ Pass
+testLoop _    []     _      = return Pass
 testLoop conf (c:cs) dyntbl = do
     res <- test conf c dyntbl
     case res of
@@ -74,7 +74,7 @@ test conf c dyntbl = do
                 return $ Just $ "Headers are different in " ++ B8.unpack wirehex ++ ":\n" ++ show hs ++ "\n" ++ show hs'
   where
     wirehex = wire c
-    inp = fst $ B16.decode wirehex
+    inp = B16.decodeLenient wirehex
     hs = headers c
 
 -- | Printing 'HeaderList'.
